indexing
	description	: "Objects that ..."

	rcsfile		: "$RCSfile: erl_feature.e,v $"
	revision	: "$Revision: 1.18 $"

	author		: "$Author: fluribe $"
	date		: "$Date: 2004/07/13 17:20:50 $"

deferred class
	ERL_FEATURE

feature -- Access

	names: LIST [STRING] is
			-- Names of this feature
		deferred
		end

	first_name: STRING is
			-- First name of feature
		do
			Result := names.first
		end

	type: ERL_TYPE is
		deferred
		end

	parents: LIST [ERL_FEATURE] is
			-- Features in parent class if inherited and redefined
		deferred
		end

	effective_parent: ERL_FEATURE is
			-- Effective feature in one of the parent classes
			-- if inherited
		deferred
		end

	precondition (a_operands: TUPLE): BOOLEAN is
			-- Do `a_operands' satisfy features's precondition?
		require
			a_operands_exist: a_operands /= Void
			a_operands_valid: valid_operands (a_operands)
		deferred
		end

	postcondition: BOOLEAN is
			-- Are postconditions satisfied invoking this feature?
		require
			postcondition_check_available: can_postcondition_be_checked
		deferred
		end

feature -- Status report

	valid_operands (a_operands: TUPLE): BOOLEAN is
			-- Are `a_operands' valid operands?
		require
			a_operands_exist: a_operands /= Void
		deferred
		end

	can_postcondition_be_checked: BOOLEAN is
			-- Can postcondition be checked?
		deferred
		end

	is_deferred: BOOLEAN is
			-- Is this feature deferred?
		deferred
		end

	is_inherited: BOOLEAN is
			-- Is this feature inherited?
		deferred
		end

	is_renamed: BOOLEAN is
			-- Is this feature renamed?
		deferred
		end

	is_redefined: BOOLEAN is
			-- Is this feature redefined?
		deferred
		end

	is_obsolete: BOOLEAN is
			-- Is this feature obsolete?
		deferred
		end

	is_frozen: BOOLEAN is
			-- Is this feature frozen?
		deferred
		end

	is_selected: BOOLEAN is
			-- Is this feature selected?
		deferred
		end

	names_not_empty (a_names: LIST [STRING]): BOOLEAN is
			-- Are all names in `names' not empty?
		require
			a_names_exist: a_names /= Void
		local
			old_cursor: CURSOR
		do
			Result := True
			old_cursor := a_names.cursor
			from
				a_names.start
			until
				a_names.after or not Result
			loop
				if a_names.item.count = 0 then
					Result := False
				end
				a_names.forth
			end
			a_names.go_to (old_cursor)
		end

	erl_type_conforming_to (other: ANY): BOOLEAN is
			-- Conforms `other' to the type this feature belongs?
		require
			other_exists: other /= Void
-- TODO: `object_type' currently not working, disabled for now
--		local
--			u: ERL_UNIVERSE
--			t: ERL_TYPE
		do
			--create u
			--t := u.object_type (other)
			--Result := t.erl_conforms_to (feature_clause.type)
			-- TODO: `object_type' currently not working, disabled for now
			Result := True
		end

invariant
	names_set: names /= Void and then
				not names.is_empty and then
				names_not_empty (names)
	parents_exists: parents /= Void
	inheritance:
		((is_renamed or is_redefined or is_selected) implies is_inherited)
		and
		(is_inherited implies not parents.is_empty)
		and
		(is_redefined implies effective_parent = Void)

end -- class ERL_FEATURE
