indexing
	description	: "Objects that ..."

	rcsfile		: "$RCSfile: erl_routine.e,v $"
	revision	: "$Revision: 1.11 $"

	author		: "$Author: fluribe $"
	date		: "$Date: 2004/07/15 11:57:59 $"

deferred class
	ERL_ROUTINE

inherit
	ERL_FEATURE
		rename
			can_postcondition_be_checked as was_call_made
		redefine
			parents,
			effective_parent,
			was_call_made
		end

feature -- Access

	precondition (a_operands: TUPLE): BOOLEAN is
			-- Do `a_operands' satisfy routine's precondition?
		do
			Result := routine.precondition (a_operands)
		end

	postcondition: BOOLEAN is
			-- Are postconditions satisfied after a call was made?
		do
			Result := routine.postcondition (operands)
		end

	formal_arguments: LIST [ERL_ARGUMENT] is
			-- All formal arguments of this routine.
		deferred
		end

	operands: TUPLE is
			-- Actual operands with which this routine operates
		do
			Result := routine.operands
		end

	parents: LIST [ERL_ROUTINE] is
			-- Routine in parent class if redefined
		deferred
		end

	effective_parent: ERL_ROUTINE is
			-- Effective routine in one of the parent classes
			-- if inherited and redefined or renamed
		deferred
		end

feature -- Status report

	valid_operands (a_operands: TUPLE): BOOLEAN is
			-- Are `a_operands' valid operands, i.e. conform they
			-- to the `formal_arguments'?
		do
			Result := routine.valid_operands (a_operands)
		end

	is_deferred: BOOLEAN
			-- Is this routine deferred?

	was_call_made: BOOLEAN
			-- Was a call made?

feature -- Element change

	set_operands (args: TUPLE) is
			-- Set `operands' to `args'.
		require
			args_exist: args /= Void
			valid_args: valid_operands (args)
		do
			routine.set_operands (args)
		ensure
			operands_set: (operands /= Void implies equal (operands, args)) or
				(operands = Void implies (args = Void or else args.is_empty))
		end

feature -- Basic operations

	call is
			-- Call routine.
		require
			valid_operands_set: valid_operands (operands)
		do
			cache_old
			routine.apply
			was_call_made := True
		ensure
			a_call_was_made: was_call_made = True
		end

feature {ERL_ROUTINE} -- Implementation

	cache_old is
			-- Cache old values for possible usage in postcondition.
		deferred
		end

	routine: ROUTINE [ANY, TUPLE] is
			-- Interface to the real routine
		deferred
		end

invariant
	set_operands_are_valid: operands /= Void implies valid_operands (operands)
	routine_exists: routine /= Void
	formal_arguments_not_void: formal_arguments /= Void

end -- class ERL_ROUTINE
