indexing
	description	: "Objects that ..."

	rcsfile		: "$RCSfile: erl_type.e,v $"
	revision	: "$Revision: 1.24 $"

	author		: "$Author: fluribe $"
	date		: "$Date: 2004/07/13 17:20:50 $"

deferred class
	ERL_TYPE

feature -- Access

	name: STRING is
			-- Name of this type without any generic information
		deferred
		end

	base_class: ERL_CLASS is
			-- This type's base class
		deferred
		end

	parents: LIST [ERL_TYPE] is
			-- All parent types of this type
		deferred
		end

	actual_generics: LIST [ERL_TYPE] is
			-- Actual generic parameters of this type
		deferred
		end

	features: LIST [ERL_FEATURE] is
			-- All features of this type.
			-- Empty set not Void if this type has
			-- no features
		deferred
		end

	creation_functions: LIST [ERL_FUNCTION] is
			-- Creation funcitons of this type.
			-- This is a dirty hack extension to ERL by aleitner
		deferred
		ensure
			result_exists: Result /= Void
		end

	feature_by_name (a_name: STRING): ERL_FEATURE is
			-- Search feature with name `a_name'.
		local
			old_cursor: CURSOR
		do
			old_cursor := features.cursor
			from
				features.start
			until
				features.after or Result /= Void
			loop
				if features.item.names.there_exists (agent a_name.is_equal) then
					Result := features.item
				else
					features.forth
				end
			end
			features.go_to (old_cursor)
		end

	creation_function_by_name (a_name: STRING): ERL_FUNCTION is
			-- Search creation function with name `a_name'.
		local
			old_cursor: CURSOR
		do
			old_cursor := creation_functions.cursor
			from
				creation_functions.start
			until
				creation_functions.after or Result /= Void
			loop
				if creation_functions.item.names.there_exists (agent a_name.is_equal) then
					Result := creation_functions.item
				else
					creation_functions.forth
				end
			end
			creation_functions.go_to (old_cursor)
		end

	feature_by_index (an_index: INTEGER): ERL_FEATURE is
		require
			an_index_big_enough: an_index > 0
			an_index_small_enough: an_index <= features.count
		deferred
		end

	creation_function_by_index (an_index: INTEGER): ERL_FUNCTION is
		require
			an_index_big_enough: an_index > 0
			an_index_small_enough: an_index <= creation_functions.count
		deferred
		end

	invariants (an_object: ANY): BOOLEAN is
			-- Does `an_object' fullfil the invariants of this type?
		require
			an_object_exists: an_object /= Void
--			an_object_is_of_this_type: an_object.generator.equals (Current)
		deferred
		end

	lookup_binding (a_feature: ERL_FEATURE): ERL_FEATURE is
			-- Feature to call according to vtable, when calling
			-- `a_feature' in parent type.
		require
			a_feature_exists: a_feature /= Void
		deferred
		end

feature -- Status report

	erl_conforms_to (other: ERL_TYPE): BOOLEAN is
			-- Does this erl type conforms to `other'?
		require
			other_exists: other /= Void
		deferred
		end

	is_expanded: BOOLEAN is
			-- Is this type expanded?
		deferred
		end

	is_deferred: BOOLEAN is
			-- Is this type deferred?
		deferred
		end

	is_obsolete: BOOLEAN is
			-- Is this type obsolete?
		deferred
		end

	is_basic_type: BOOLEAN is
			-- Is this type a basic type (INTEGER,...)?
		deferred
		end

	has_actual_generics: BOOLEAN is
			-- Has this type actual generic parameters?
		deferred
		end

invariant
	name_not_empty: name /= Void and then
					not name.is_empty
	base_class_exists: base_class /= Void
			-- disabled for now, since parents dont work yet
-- 	parents_not_empty: parents /= Void and then
-- 						(not name.is_equal ("ANY") implies
-- 						not parents.is_empty)
						-- at least `ANY' should be in the parents.
-- 	actual_generics_exist: actual_generics /= Void
-- 	actual_generics_not_empty: base_class.is_generic implies
-- 								not actual_generics.is_empty
--	features_exist: features /= Void

end -- class TYPE
