indexing
	description	: "Objects that ..."

	rcsfile		: "$RCSfile: erl_attribute_imp.e,v $"
	revision	: "$Revision: 1.11 $"

	author		: "$Author: fluribe $"
	date		: "$Date: 2004/07/15 11:58:00 $"

class
	ERL_ATTRIBUTE_IMP

inherit
	ERL_ATTRIBUTE

	ERL_QUERY_IMP
		rename
			result_type as attribute_type
		redefine
			make,
			attribute_type
		end

create
	make

feature {NONE} -- Initialization

	make (a_names: LIST [STRING]) is
			-- Initialize `Current'.
		do
			Precursor {ERL_QUERY_IMP} (a_names)
			create internal
		ensure then
			internal_exists: internal /= Void
		end

feature -- Access

	attribute_type: ERL_TYPE is
			-- See `{ERL_QUERY}.result_type' for more details.
		local
			field_type: INTEGER
			o_type: INTEGER
		do
			o_type := internal.dynamic_type (object)
			field_type := internal.field_static_type_of_type (position, o_type)
			if is_deferred_type (field_type) then
				create {ERL_TYPE_IMP} Result.make_for_name (internal.type_name_of_type (field_type))
			else
				create {ERL_TYPE_IMP} Result.make (internal.new_instance_of (field_type))
			end
		end

	item: ANY is
			-- See `{ERL_ATTRIBUTE}.item' for more details.
		do
			check
				position_positive: position > 0
			end
			Result := internal.field (position, object)
		end

	parents: LIST [ERL_QUERY] is
			-- See `{ERL_ATTRIBUTE}.parents' for more details.
		do
			--| FIXME compiler / runtime support needed.
			-- create {ARRAY [ERL_QUERY]} Result.make (1, 1)
		end

	effective_parent: ERL_QUERY is
			-- See `{ERL_ATTRIBUTE}.effective_parent' for more details.
		do
			--| FIXME compiler / runtime support needed.
		end

	precondition (a_operands: TUPLE): BOOLEAN is
			-- See `{ERL_FEATURE}.precondition' for more details.
		do
			--| FIXME compiler / runtime support needed
			Result := True
		end

	postcondition: BOOLEAN is
			-- See `{ERL_FEATURE}.postcondition' for more details.
		do
			--| FIXME compiler / runtime support needed
			Result := True
		end

feature -- Status report

	valid_operands (a_operands: TUPLE): BOOLEAN is
			-- Are `a_operands' valid operands?
		do
			Result := erl_type_conforming_to (a_operands @ 1)
		end

	can_postcondition_be_checked: BOOLEAN is
			-- Can postcondition be checked?
		do
			--| FIXME compiler / runtime support needed
			Result := True
		end

feature {ERL_TYPE} -- Status setting

	set_position (a_position: INTEGER) is
			-- Set `position' to `a_position'.
		require
			a_position_positive: a_position > 0
		do
			position := a_position
		ensure
			position_set: position = a_position
		end

feature -- Element change

	replace (a_value: ANY) is
			-- See `{ERL_ATTRIBUTE}.replace' for more details.
		local
			ft: INTEGER
			intref: INTEGER_REF
			boolref: BOOLEAN_REF
			pref: POINTER_REF
			chref: CHARACTER_REF
			realref: REAL_REF
			dref: DOUBLE_REF
			int8ref: INTEGER_8_REF
			int16ref: INTEGER_16_REF
			int64ref: INTEGER_64_REF
		do
			check
				position_positive: position > 0
			end
			ft := internal.field_type (position, object)
			if ft = internal.Reference_type	then
				internal.set_reference_field (position, object, a_value)
			elseif ft =	internal.Pointer_type then
				pref ?= a_value
				internal.set_pointer_field (position, object, pref.item)
			elseif ft =	internal.Character_type then
				chref ?= a_value
				internal.set_character_field (position, object, chref.item)
			elseif ft =	internal.Boolean_type then
				boolref ?= a_value
				internal.set_boolean_field (position, object, boolref.item)
			elseif ft =	internal.Integer_type then
				intref ?= a_value
				internal.set_integer_field (position, object, intref.item)
			elseif ft =	internal.Real_type then
				realref ?= a_value
				internal.set_real_field (position, object, realref.item)
			elseif ft =	internal.Double_type then
				dref ?= a_value
				internal.set_double_field (position, object, dref.item)
			elseif ft =	internal.Integer_8_type then
				int8ref ?= a_value
				internal.set_integer_8_field (position, object, int8ref.item)
			elseif ft =	internal.Integer_16_type then
				int16ref ?= a_value
				internal.set_integer_16_field (position, object, int16ref.item)
			elseif ft =	internal.Integer_64_type then
				int64ref ?= a_value
				internal.set_integer_64_field (position, object, int64ref.item)
			end
		end

feature {NONE} -- Implementation

	is_deferred_type (type_id: INTEGER): BOOLEAN is
			-- Is the type of `type_id' deferred?
		local
			o: ANY
			already: BOOLEAN
		do
			if not already then
				o := internal.new_instance_of (type_id)
				Result := False
			else
				Result := True
			end
		rescue
			already := True
			retry
		end

	internal: INTERNAL
			-- Native reflection support.

	position: INTEGER
			-- Position of this attribute in type.

end -- class ERL_ATTRIBUTE_IMP
