indexing

	description: "Interpreter for Eiffel systems"
	date: "$Date: 2004/08/24 10:36:34$"

class TS_RT_INTERPRETER

inherit

	GERL_SHARED_UNIVERSE
	EXCEPTIONS
	TS_RT_REQUEST_PARSER
	ARGUMENTS

create

	execute

feature -- Processing

	execute is
			-- Execute test client
		local
			nb: INTEGER
			log_filename: STRING
		do
			nb := argument_count
			if nb < 1 then
				print_and_flush ("usage: generated_test log_filename%N")
				die (1)
			end
			log_filename := argument (1)
			create trace_file.make_open_append (log_filename)
			if not trace_file.is_open_write then
				print_and_flush ("error: could not open log file%N")
				die (1)
			end
			create object_repository.make (1, 1)
			put_object (Void)
			main_loop
			trace_file.put_string ("</session>%N")
			trace_file.close
		rescue
			print ("error: internal: exiting.%N")
			if trace_file.is_open_write then
				trace_file.put_string ("<error type='internal'/>%N")
				trace_file.put_string ("</session>%N")
				trace_file.close
			end
		end

	main_loop is
			-- Main loop
		local
			input_line: STRING
		do
			trace_file.put_string ("<session>%N")
			from
			until
				should_quit or io.input.end_of_file
			loop
				io.input.read_line
				input_line := io.input.last_string
				if input_line = Void or else input_line.count = 0 then
					print_and_flush ("error: syntax: empty input%N")
				else
					set_input (input_line)
					parse
				end
				print_and_flush ("done:%N")
			end
		end

feature {NONE} -- Handlers

	report_invoke_request (a_type_name: STRING;
								  a_feature_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Invoke feature `a_type_name'.`a_feature_name'.
		local
			type: ERL_TYPE
			routine: ERL_ROUTINE
		do
			type := universe.type_by_name (a_type_name)
			if type = Void then
				print_and_flush ("error:unknown type '" + a_type_name + "'%N")
			else
				routine ?= type.feature_by_name (a_feature_name)
				if routine = Void then
					print_and_flush ("error:unknown feature '" + a_feature_name + "'%N")
				else
					trace_file.put_string ("<try_call type='" + routine.type.name + "' feature='" + routine.first_name + "'>%N")
					if an_operand_list.count > 0 then
						routine.set_operands (operands (an_operand_list, routine.operands, True))
					end
					call_routine (routine, 0)
					trace_file.put_string ("</try_call>%N")
				end
			end
		end

	report_precondition_invoke_request (a_type_name: STRING;
								  a_feature_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Evaluate the precondition of feature `a_type_name'.`a_feature_name'.
		local
			type: ERL_TYPE
			routine: ERL_ROUTINE
		do
			type := universe.type_by_name (a_type_name)
			if type = Void then
				print_and_flush ("error:unknown type '" + a_type_name + "'%N")
			else
				routine ?= type.feature_by_name (a_feature_name)
				if routine = Void then
					print_and_flush ("error:unknown feature '" + a_feature_name + "'%N")
				else
					trace_file.put_string ("<try_call_precondition type='" + routine.type.name + "' feature='" + routine.first_name + "'>%N")
					call_precondition_of_routine (routine, operands (an_operand_list, routine.operands, True))
					trace_file.put_string ("</try_call_precondition>%N")
				end
			end
		end

	report_create_request (a_type_name: STRING;
								  a_creation_procedure_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Create object of type `a_type_name' using creation procedure `a_creation_procedure_name'.
		local
			type: ERL_TYPE
			function: ERL_FUNCTION
		do
			type := universe.type_by_name (a_type_name)
			if type = Void then
				print_and_flush ("error:unknown type '" + a_type_name + "'%N")
			else
				function ?= type.creation_function_by_name (a_creation_procedure_name)
				if function = Void then
					print_and_flush ("error:unknown creation procedure '" + a_creation_procedure_name + "'%N")
				else
					trace_file.put_string ("<try_create type='" + function.type.name + "' feature='" + function.first_name + "'>%N")
					if an_operand_list.count > 1 then
						function.set_operands (operands (an_operand_list, function.operands, False))
					end
						check
							an_operand_list_count_valid: an_operand_list.count > 0
							valid_target_slot: an_operand_list.first.is_reference
						end
					call_routine (function, an_operand_list.first.index)
					trace_file.put_string ("</try_create>%N")
				end
			end
		end

	report_precondition_create_request (a_type_name: STRING;
								  a_creation_procedure_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Invoke precondition of creation procedure `a_type_name'.`a_creation_procedure_name'.
			-- TODO: really bad name
		local
			type: ERL_TYPE
			function: ERL_FUNCTION
		do
			type := universe.type_by_name (a_type_name)
			if type = Void then
				print_and_flush ("error:unknown type '" + a_type_name + "'%N")
			else
				function ?= type.creation_function_by_name (a_creation_procedure_name)
				if function = Void then
					print_and_flush ("error:unknown creation procedure '" + a_creation_procedure_name + "'%N")
				else
					trace_file.put_string ("<try_create_precondition type='" + function.type.name + "' feature='" + function.first_name + "'>%N")
					call_precondition_of_routine (function, operands (an_operand_list, function.operands, True))
					trace_file.put_string ("</try_create_precondition>%N")
				end
			end
		end

	report_quit_request is
			-- Set `should_quit' to `True'
		do
			should_quit := True
		end

feature

	print_result (an_object: ANY; a_static_type: ERL_TYPE) is
			-- Print `an_object' of type `a_type'.
		require
			a_static_type_not_void: a_static_type /= Void
		local
			index: INTEGER
			dynamic_type: ERL_TYPE
		do
			dynamic_type := universe.type_by_object (an_object)
			if dynamic_type = Void then
				-- Workaround for when the dynamic type is not introspectable
				dynamic_type := a_static_type
			end
			trace_file.put_string ("<result static_type=%"")
			trace_file.put_string (a_static_type.name)
			trace_file.put_string ("%" dynamic_type=%"")
			trace_file.put_string (dynamic_type.name)
			trace_file.put_string ("%">%N")
			print ("result:")
			if a_static_type.is_basic_type then
				print (a_static_type.name)
				print ("%"")
				print (an_object)
				print ("%"")
			else
				index := object_index (an_object)
				print ("REFERENCE%"")
				print (index)
				print ("%"%"")
				print (dynamic_type.name)
				print ("%"")
				trace_file.put_string ("<slot>")
				trace_file.put_string (index.out)
				trace_file.put_string ("</slot>")
			end
			log_instance (an_object)
			print_and_flush ("%N")
			trace_file.put_string ("</result>%N")
		end

	print_multi_line_value_start_tag is
			-- Print the start tag for a multi line value.
		do
			print (multi_line_value_start_tag)
			print_and_flush ("%N")
		end

	print_multi_line_value_end_tag is
			-- Print the start tag for a multi line value.
		do
			print (multi_line_value_end_tag)
			print_and_flush ("%N")
		end

feature {NONE} -- Execution

	last_call_status: STRING
	last_exception_trace: STRING

	call_routine (a_routine: ERL_ROUTINE; a_target_slot: INTEGER) is
			-- Call routine `a_routine'.
		require
			a_routine_not_void: a_routine /= Void
			a_target_slot_positive: a_target_slot >= 0
		local
			failed: BOOLEAN
			function: ERL_FUNCTION
			l_exception: INTEGER
			l_recipient_name: STRING
			l_class_name: STRING
			l_tag_name: STRING
			l_meaning: STRING
			l_exception_trace: STRING
		do
			if not failed then
				last_exception_trace := Void
				print_multi_line_value_start_tag
				a_routine.call
				print_multi_line_value_end_tag
				trace_file.put_string ("<call_result type='success'/>%N")
				last_call_status := "%"pass%""
			end
			print_and_flush ("status:" + last_call_status + "%N")
			if last_exception_trace /= Void then
				print_multi_line_value_start_tag
				print_and_flush (last_exception_trace)
				print_multi_line_value_end_tag
			end
			if not failed then
				function ?= a_routine
				if function /= Void then
					if not function.result_type.is_basic_type then
						if a_target_slot = 0 then
							put_object (function.last_result)
						else
							put_object_at (function.last_result, a_target_slot)
						end
					end
					print_result (function.last_result, function.result_type)
				end
			end
		ensure
			last_call_status_not_void: last_call_status /= Void
		rescue
			print_multi_line_value_end_tag
			l_exception := exception
			l_tag_name := tag_name
			l_recipient_name := recipient_name
			l_class_name := class_name
			l_meaning := meaning (l_exception)
			l_exception_trace := exception_trace

			if l_recipient_name = Void then
				l_recipient_name := ""
			end
			if l_class_name = Void then
				-- TODO: (aleitner)I am not sure why `class_name' can be Void;
				-- I sent an email to Manu to ask for clarification.
				-- For now we assume if `class_name' is Void, the correct class is the
				-- class of the feature under test.
				l_class_name := a_routine.type.base_class.name
			end
			if l_tag_name = Void then
				l_tag_name := ""
			end
			if l_meaning = Void then
				l_meaning := ""
			end

			last_call_status := "%"fail%"%"" + l_exception.out + "%"%"" + l_recipient_name + "%"%"" + l_class_name + "%"%"" + l_tag_name + "%""
			last_exception_trace := l_exception_trace
			trace_file.put_string ("<call_result type='exception'>%N")
			trace_file.put_string ("<meaning value='" + l_meaning + "'/>%N")
			trace_file.put_string ("<tag value='" + l_tag_name + "'/>%N")
			trace_file.put_string ("<recipient value='" + l_recipient_name + "'/>%N")
			trace_file.put_string ("<class value='" + l_class_name + "'>%N")
			trace_file.put_string ("<exception_trace>%N<![CDATA[%N")
			trace_file.put_string (l_exception_trace)
			trace_file.put_string ("]]>%N</exception_trace>%N")
			trace_file.put_string ("</call_result>%N")
			failed := True
			if l_exception = Class_invariant then
				-- A class invariant cannot be recovered from since we
				-- don't know how many and what objects are now invalid
				should_quit := True
			end
			retry
		end

	call_precondition_of_routine (a_routine: ERL_ROUTINE; a_operands: TUPLE) is
			-- Call routine `a_routine'.
		require
			a_routine_not_void: a_routine /= Void
			a_operands_not_void: a_operands /= Void
		local
			failed: BOOLEAN
			l_exception: INTEGER
			l_recipient_name: STRING
			l_class_name: STRING
			l_tag_name: STRING
			l_meaning: STRING
			l_exception_trace: STRING
			value: BOOLEAN
		do
			if not failed then
				last_exception_trace := Void
				print_multi_line_value_start_tag
				value := a_routine.precondition (a_operands)
				print_multi_line_value_end_tag
				trace_file.put_string ("<precondition_call_result type='success'/>%N")
				last_call_status := "%"pass%""
			end
			print_and_flush ("status:" + last_call_status + "%N")
			if last_exception_trace /= Void then
				print_multi_line_value_start_tag
				print (last_exception_trace)
				print_multi_line_value_end_tag
			end
			if not failed then
				print_result (value, universe.type_by_name ("BOOLEAN"))
			end
		ensure
			last_call_status_not_void: last_call_status /= Void
		rescue
			print_multi_line_value_end_tag
			l_exception := exception
			l_tag_name := tag_name
			l_recipient_name := recipient_name
			l_class_name := class_name
			l_meaning := meaning (l_exception)
			l_exception_trace := exception_trace

			if l_recipient_name = Void then
				l_recipient_name := ""
			end
			if l_class_name = Void then
				-- TODO: (aleitner)I am not sure why `class_name' can be Void;
				-- I sent an email to Manu to ask for clarification.
				-- For now we assume if `class_name' is Void, the correct class is the
				-- class of the feature under test.
				l_class_name := a_routine.type.base_class.name
			end
			if l_tag_name = Void then
				l_tag_name := ""
			end
			if l_meaning = Void then
				l_meaning := ""
			end

			last_call_status := "%"fail%"%"" + l_exception.out + "%"%"" + l_recipient_name + "%"%"" + l_class_name + "%"%"" + l_tag_name + "%""
			last_exception_trace := l_exception_trace
			trace_file.put_string ("<precondition_call_result type='exception'>%N")
			trace_file.put_string ("<meaning value='" + l_meaning + "'/>%N")
			trace_file.put_string ("<tag value='" + l_tag_name + "'/>%N")
			trace_file.put_string ("<recipient value='" + l_recipient_name + "'/>%N")
			trace_file.put_string ("<class value='" + l_class_name + "'>%N")
			trace_file.put_string ("<exception_trace>%N<![CDATA[%N")
			trace_file.put_string (l_exception_trace)
			trace_file.put_string ("]]>%N</exception_trace>%N")
			trace_file.put_string ("</precondition_call_result>%N")
			failed := True
			if l_exception = Class_invariant then
				-- A class invariant cannot be recovered from since we
				-- don't know how many and what objects are now invalid
				should_quit := True
			end
			retry
		end

feature {NONE} -- Repository

	object_repository: ARRAY [ANY]
			-- Object repository

	put_object (an_object: ANY) is
		local
			i: INTEGER
		do
			i := object_index (an_object)
			if i = 0 then
				object_repository.force (an_object, object_repository.upper + 1)
			end
		end

	put_object_at (an_object: ANY; a_target_slot: INTEGER) is
		require
			a_target_slot_positive: a_target_slot > 0
		do
			object_repository.force (an_object, a_target_slot)
		end

	has_object (an_object: ANY): BOOLEAN is
		do
			Result := object_index (an_object) /= 0
		end

	object_index (an_object: ANY): INTEGER is
			-- Index of `an_object' in `object_repository', `0' if it
			-- doesn't occur.
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > object_repository.count or Result > 0
			loop
				if object_repository.item (i) = an_object then
					Result := i
				else
					i := i + 1
				end
			end
		ensure
			index_positive: Result >= 0
		end

	object (an_index: INTEGER): ANY is
		require
			valid_index: an_index > 0
		do
			if an_index <= object_repository.count then
				Result := object_repository.item (an_index)
			end
		ensure
			an_index = 1 implies Result = Void
		end

	operands (an_operand_list: LIST [TS_RT_OPERAND]; a_tuple_prototype: TUPLE; keep_first: BOOLEAN): TUPLE is
			-- Create an operand tuple from `object_repository' based on `an_operand_list'.
			-- If `keep_first' is `False' then ignore the first item in `an_operand_list'.
		require
			an_operand_list_not_void: an_operand_list /= Void
		local
			old_cursor: CURSOR
			op: TS_RT_OPERAND
			obj: ANY
			result_index: INTEGER
		do
			trace_file.put_string ("<operands>%N")
			if a_tuple_prototype /= Void then
				old_cursor := an_operand_list.cursor
				from
					Result := a_tuple_prototype.twin
					result_index := 1
					an_operand_list.start
					if not keep_first and not an_operand_list.off then
						an_operand_list.forth
					end
				until
					an_operand_list.off
				loop
					op := an_operand_list.item
					obj := op.value (object_repository)
					Result.put (obj, result_index)
					trace_file.put_string ("<operand>")
					if op.is_reference then
						trace_file.put_string ("<slot>")
						trace_file.put_string (op.index.out)
						trace_file.put_string ("</slot>")
					end
					log_instance (obj)
					trace_file.put_string ("</operand>")
					an_operand_list.forth
					result_index := result_index + 1
				end
				an_operand_list.go_to (old_cursor)
			end
			trace_file.put_string ("</operands>%N")
		ensure
			result_not_void: Result /= Void
		end

feature {NONE} -- Logging

	trace_file: PLAIN_TEXT_FILE
			-- Trace file

	log_instance (an_object: ANY) is
		do
			trace_file.put_string ("<instance<![CDATA[%N")
			if an_object = Void then
				trace_file.put_string ("Void%N")
			else
				trace_file.put_string (an_object.tagged_out)
			end
			trace_file.put_string ("]]>%N</instance>%N")
		end

feature {NONE} -- Implementation

	should_quit: BOOLEAN
			-- Should main loop quit?

invariant

	object_repository_not_void: object_repository /= Void
	object_repository_lower: object_repository.lower = 1
	object_repository_upper: object_repository.lower > 1
	object_repository_not_empty: object_repository.count > 0
	object_repository_has_void: object_repository.item (1) = Void
	trace_file_not_void: trace_file /= Void
	trace_file_open_write: trace_file.is_open_write

end
