indexing

	description: "Operands"
	date: "$Date: 2004/08/24 10:36:34$"

class TS_RT_OPERAND

create

	make_reference,
	make_constant

feature {NONE} -- Initialization

	make_reference (an_index: INTEGER) is
		require
			an_index_positive: an_index > 0
		do
			index := an_index
		ensure
			is_reference: is_reference
		end

	make_constant (a_type_name: STRING; a_constant: ANY) is
		require
			a_type_name_not_void: a_type_name /= Void
			a_type_name_not_empty: a_type_name.count > 0
			a_constant_not_void: a_constant /= Void
		do
			type_name := a_type_name
			constant := a_constant
		ensure
			type_name_set: type_name = a_type_name
			constant_set: constant = a_constant
			not_reference: not is_reference
		end

feature -- Status

	is_reference: BOOLEAN is
		do
			Result := index > 0
		end

feature -- Access

	value (a_repository: ARRAY [ANY]): ANY is
		require
			a_repository_not_void: a_repository /= Void
		do
			if is_reference then
				Result := a_repository.item (index)
			else
				Result := constant
			end
		end

	to_text: STRING is
		local
			constant_text: STRING
		do
			if is_reference then
				Result := ("REFERENCE%"").twin
				Result.append_string (index.out)
				Result.append_character ('"')
			else
				Result := type_name.twin
				Result.append_character ('"')
				constant_text := constant.out
				-- TODO: Since the parser doesnt understand escape codes
				-- we need to make sure not to send new lines.
				-- Eventually characters should be escaped according to ETL rules
				-- and the parser should be made aware of this.
				constant_text.prune_all ('%N')
				constant_text.prune_all ('%R')
				Result.append_string (constant_text)
				Result.append_character ('"')
			end
		ensure
			text_not_void: Result /= Void
			text_not_empty: Result.count > 0
		end

	index: INTEGER
			-- Only valid when `is_reference'

	constant: ANY
			-- Only valid when `not is_reference'

	type_name: STRING
			-- Only valid when `not is_reference'

	integer_value: INTEGER is
			-- Value of this operand converted to an integer; TODO: query
			-- hides errors, come up with better invariant that ensures a
			-- valid `type_name' and a valid type of `constant' in cases
			-- where the operand does not represent a reference.
		local
			boolean_ref: BOOLEAN_REF
			integer_ref: INTEGER_REF
			integer_8_ref: INTEGER_8_REF
			integer_16_ref: INTEGER_16_REF
			integer_64_ref: INTEGER_64_REF
			character_ref: CHARACTER_REF
			wide_character_ref: WIDE_CHARACTER_REF
			double_ref: DOUBLE_REF
			real_ref: REAL_REF
		do
			if is_reference then
				Result := index
			elseif type_name.is_equal ("BOOLEAN") then
				boolean_ref ?= constant
				if boolean_ref /= Void and then boolean_ref.item then
					Result := 1
				else
					Result := 0
				end
			elseif type_name.is_equal ("INTEGER") then
				integer_ref ?= constant
				if integer_ref /= Void then
					Result := integer_ref.item
				end
			elseif type_name.is_equal ("INTEGER_8") then
				integer_8_ref ?= constant
				if integer_8_ref /= Void then
					Result := integer_8_ref.item
				end
			elseif type_name.is_equal ("INTEGER_16") then
				integer_16_ref ?= constant
				if integer_16_ref /= Void then
					Result := integer_16_ref.item
				end
			elseif type_name.is_equal ("INTEGER_64") then
				integer_64_ref ?= constant
				if integer_64_ref /= Void then
					Result := integer_64_ref.to_integer
				end
			elseif type_name.is_equal ("CHARACTER") then
				character_ref ?= constant
				if character_ref /= Void then
					Result := character_ref.code
				end
			elseif type_name.is_equal ("WIDE_CHARACTER") then
				wide_character_ref ?= constant
				if wide_character_ref /= Void then
					Result := wide_character_ref.code
				end
			elseif type_name.is_equal ("DOUBLE") then
				double_ref ?= constant
				if double_ref /= Void then
					Result := double_ref.rounded
				end
			elseif type_name.is_equal ("REAL") then
				real_ref ?= constant
				if real_ref /= Void then
					Result := real_ref.rounded
				end
			else
				Result := 0
			end
		end


invariant

	constant_means_valid_type: not is_reference implies type_name /= Void and then type_name.count > 0
	constant: not is_reference implies constant /= Void

end
