indexing

	description: "Request language parser"
	author: "Andreas Leitner"
	date: "$Date: 2003/06/13 06:27:44 $"
	revision: "$Revision: 1.11 $"

deferred class TS_RT_REQUEST_PARSER

inherit

	TS_RT_PARSER

feature -- Status

	last_operand_list: LIST [TS_RT_OPERAND]

feature -- Parsing

	parse is
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		do
			parse_command_type
			if has_error then
				print_and_flush ("error:syntax:invalid request%N")
			else
				if last_string.is_equal ("quit") then
					report_quit_request
				elseif end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					if last_string.is_equal ("create") then
						parse_create_request
					elseif last_string.is_equal ("invoke") then
						parse_invoke_request
					elseif last_string.is_equal ("pre_create") then
						parse_precondition_create_request
					elseif last_string.is_equal ("pre_invoke") then
						parse_precondition_invoke_request
					else
						print_and_flush ("error:syntax:unkown request '" + last_string + "'%N")
					end
				end
			end
		end

	parse_create_request is
			-- Parse a create request.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			type_name: STRING
			creation_procedure_name: STRING
		do
			parse_value
			if has_error then
				print_and_flush ("error:syntax:expected type name%N")
			elseif end_of_input then
				print_and_flush ("error:syntax:didn't expect end of input%N")
			else
				type_name := last_string
				parse_token (".")
				if has_error then
					print_and_flush ("error:syntax:expected '.'%N")
				elseif end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					parse_value
					if has_error then
						print_and_flush ("error:syntax:expected creation procedure name%N")
					elseif end_of_input then
						print_and_flush ("error:syntax:didn't expect end of input%N")
					else
						creation_procedure_name := last_string
						parse_token (" ")
						if has_error then
							print_and_flush ("error:syntax:expected ' '%N")
						elseif end_of_input then
							print_and_flush ("error:syntax:didn't expect end of input%N")
						else
							parse_operands
							if has_error then
								print_and_flush ("error:syntax:expected operands%N")
							else
								report_create_request (type_name,
															  creation_procedure_name,
															  last_operand_list)
							end
						end
					end
				end
			end
		end

	parse_precondition_create_request is
			-- Parse a create request.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			type_name: STRING
			creation_procedure_name: STRING
		do
			parse_value
			if has_error then
				print_and_flush ("error:syntax:expected type name%N")
			elseif end_of_input then
				print_and_flush ("error:syntax:didn't expect end of input%N")
			else
				type_name := last_string
				parse_token (".")
				if has_error then
					print_and_flush ("error:syntax:expected '.'%N")
				elseif end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					parse_value
					if has_error then
						print_and_flush ("error:syntax:expected creation procedure name%N")
					elseif end_of_input then
						print_and_flush ("error:syntax:didn't expect end of input%N")
					else
						creation_procedure_name := last_string
						parse_token (" ")
						if has_error then
							print_and_flush ("error:syntax:expected ' '%N")
						elseif end_of_input then
							print_and_flush ("error:syntax:didn't expect end of input%N")
						else
							parse_operands
							if has_error then
								print_and_flush ("error:syntax:expected operands%N")
							else
								report_precondition_create_request (type_name,
																				creation_procedure_name,
																				last_operand_list)
							end
						end
					end
				end
			end
		end

	parse_invoke_request is
			-- Parse an invoke request.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			type_name: STRING
			feature_name: STRING
		do
			parse_value
			if has_error then
				print_and_flush ("error:syntax:expected type name%N")
			elseif end_of_input then
				print_and_flush ("error:syntax:didn't expect end of input%N")
			else
				type_name := last_string
				parse_token (".")
				if has_error then
					print_and_flush ("error:syntax:expected '.'%N")
				elseif end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					parse_value
					if has_error then
						print_and_flush ("error:syntax:expected feature name%N")
					elseif end_of_input then
						print_and_flush ("error:syntax:didn't expect end of input%N")
					else
						feature_name := last_string
						parse_token (" ")
						if has_error then
							print_and_flush ("error:syntax:expected ' '%N")
						elseif end_of_input then
							print_and_flush ("error:syntax:didn't expect end of input%N")
						else
							parse_operands
							if has_error then
								print_and_flush ("error:syntax:expected operands%N")
							else
								report_invoke_request (type_name, feature_name, last_operand_list)
							end
						end
					end
				end
			end
		end

	parse_precondition_invoke_request is
			-- Parse an invoke request.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			type_name: STRING
			feature_name: STRING
		do
			parse_value
			if has_error then
				print_and_flush ("error:syntax:expected type name%N")
			elseif end_of_input then
				print_and_flush ("error:syntax:didn't expect end of input%N")
			else
				type_name := last_string
				parse_token (".")
				if has_error then
					print_and_flush ("error:syntax:expected '.'%N")
				elseif end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					parse_value
					if has_error then
						print_and_flush ("error:syntax:expected feature name%N")
					elseif end_of_input then
						print_and_flush ("error:syntax:didn't expect end of input%N")
					else
						feature_name := last_string
						parse_token (" ")
						if has_error then
							print_and_flush ("error:syntax:expected ' '%N")
						elseif end_of_input then
							print_and_flush ("error:syntax:didn't expect end of input%N")
						else
							parse_operands
							if has_error then
								print_and_flush ("error:syntax:expected operands%N")
							else
								report_precondition_invoke_request (type_name, feature_name, last_operand_list)
							end
						end
					end
				end
			end
		end

	parse_operands is
			-- Parse operands and make result available via `last_operand_list'.
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		do
			parse_token ("(")
			if has_error then
				print_and_flush ("error:syntax:expected '('%N")
			elseif end_of_input then
				print_and_flush ("error:syntax:didn't expect end of input%N")
			else
				from
					create {ARRAYED_LIST [TS_RT_OPERAND]} last_operand_list.make (10)
				until
					has_error or end_of_input
				loop
					parse_operand
					if not has_error then
						last_operand_list.extend (last_operand)
					end
				end
				has_error := False
				if end_of_input then
					print_and_flush ("error:syntax:didn't expect end of input%N")
				else
					parse_token (")")
				end
			end
		end

feature {NONE} -- Handlers

	report_invoke_request (a_type_name: STRING;
								  a_feature_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
		require
			a_type_name_not_void: a_type_name /= Void
			a_type_name_not_empty: a_type_name.count > 0
			a_feature_name_not_void: a_feature_name /= Void
			a_feature_name_not_empty: a_feature_name.count > 0
			an_operand_list_not_void: an_operand_list /= Void
			operands_not_void: not an_operand_list.has (Void)
			-- Report an "invoke" request.
		deferred
		end

	report_create_request (a_type_name: STRING;
								  a_creation_procedure_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Report a create request
		require
			a_creation_procedure_name_not_void: a_creation_procedure_name /= Void
			a_creation_procedure_name_not_empty: a_creation_procedure_name.count > 0
			an_operand_list_not_void: an_operand_list /= Void
			operands_not_void: not an_operand_list.has (Void)
		deferred
		end

	report_precondition_invoke_request (a_type_name: STRING;
								  a_feature_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
		require
			a_type_name_not_void: a_type_name /= Void
			a_type_name_not_empty: a_type_name.count > 0
			a_feature_name_not_void: a_feature_name /= Void
			a_feature_name_not_empty: a_feature_name.count > 0
			an_operand_list_not_void: an_operand_list /= Void
			operands_not_void: not an_operand_list.has (Void)
			-- Report an "evaluate precondition of feature" request.
		deferred
		end

	report_precondition_create_request (a_type_name: STRING;
								  a_creation_procedure_name: STRING;
								  an_operand_list: LIST [TS_RT_OPERAND]) is
			-- Report an "evaluate precondition of creation procedure" request
		require
			a_creation_procedure_name_not_void: a_creation_procedure_name /= Void
			a_creation_procedure_name_not_empty: a_creation_procedure_name.count > 0
			an_operand_list_not_void: an_operand_list /= Void
			operands_not_void: not an_operand_list.has (Void)
		deferred
		end

	report_quit_request is
			-- Report a quit request
		deferred
		end
		
feature {NONE} -- Printing

	print_and_flush (a_text: STRING) is
			-- Print `a_text' and flush output stream.
		require
			a_text_not_void: a_text /= Void
		do
			print (a_text)
			io.output.flush
		end

end
