indexing

	description: "Response language parser"
	author: "Andreas Leitner"
	date: "$Date: 2003/06/13 06:27:44 $"
	revision: "$Revision: 1.11 $"

deferred class TS_RT_RESPONSE_PARSER

inherit

	TS_RT_PARSER
	EXCEPTIONS

feature -- Parsing

	parse_response is
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			operand: TS_RT_OPERAND
			type_name: STRING
		do
			parse_command_type
			if has_error then
				report_syntax_error
			else
				if last_string.is_equal ("done") then
					report_done_response
				elseif last_string.is_equal ("result") then
					if end_of_input then
						report_syntax_error
					else
						parse_operand
						if has_error then
							report_syntax_error
						else
							operand := last_operand
							if operand.is_reference then
								if end_of_input then
									report_syntax_error
								else
									parse_value
									if has_error or last_string.count = 0 then
										report_syntax_error
									else
										type_name := last_string
										report_result_response (last_operand, type_name)
									end
								end
							else
								report_result_response (last_operand, last_operand.type_name)
							end
						end
					end
				elseif last_string.is_equal ("error") then
					report_error_response
				elseif last_string.is_equal ("status") then
					if end_of_input then
						report_syntax_error
					else
						parse_status
					end
				end
			end
		end

	parse_status is
		require
			input: input /= Void
			not_end_of_input: not end_of_input
			no_error: not has_error
		local
			l_exception_code: INTEGER
			l_recipient_name: STRING
			l_class_name: STRING
			l_tag_name: STRING
		do
			parse_value
			if has_error then
				report_syntax_error
			else
				if last_string.is_equal ("pass") then
					report_status_passed_response
				else
					if end_of_input then
						report_syntax_error
					else
						parse_value
						if has_error or end_of_input then
							report_syntax_error
						else
							if not last_string.is_integer or else not valid_code (last_string.to_integer) then
								report_syntax_error
							else
								l_exception_code := last_string.to_integer
								parse_value
								if (has_error or end_of_input) or else last_string.count = 0 then
									report_syntax_error
								else
									l_recipient_name := last_string
									parse_value
									if (has_error or end_of_input) or else last_string.count = 0 then
										report_syntax_error
									else
										l_class_name := last_string
										parse_value
										if has_error then
											report_syntax_error
										else
											l_tag_name := last_string
											report_status_failed_response (l_exception_code, l_recipient_name, l_class_name, l_tag_name)
										end
									end
								end
							end
						end
					end
				end
			end
		end

feature

	report_syntax_error is
		deferred
		end

	report_done_response is
		deferred
		end

	report_result_response (a_result: TS_RT_OPERAND; a_type_name: STRING) is
		require
			a_result_not_void: a_result /= Void
			a_type_name_not_void: a_type_name /= Void
			a_type_name_not_empty: a_type_name.count > 0
		deferred
		end

	report_status_passed_response is
		deferred
		end

	report_status_failed_response (an_exception_code: INTEGER;
											 a_recipient_name: STRING;
											 a_class_name: STRING;
											 a_tag_name: STRING) is
		require
			an_exception_code_valid: valid_code (an_exception_code)
			a_recipient_name_not_void: a_recipient_name /= Void
			a_recipient_name_not_empty: a_recipient_name.count > 0
			a_class_name_not_void: a_class_name /= Void
			a_class_name_not_empty: a_class_name.count > 0
			a_tag_name_not_void: a_tag_name /= Void
		deferred
		end

	report_error_response is
		deferred
		end

end
