
indexing

	description: "Ancestor of all test classes"
	date: "$Date: 2004/08/17 09:09:23$"

deferred class TS_RT_TEST

inherit

	EXCEPTIONS
	TS_RT_SHARED_DEBUG_MODE

feature -- Run

	run (a_context: TSGEN_CONTEXT; a_file: PLAIN_TEXT_FILE): BOOLEAN is
			-- Call test feature call in `a_file' in context `a_context'.
			-- Set exceptions.
		local
			failure: INTEGER
			precond_violated: BOOLEAN
		do
			if failure < 1 then
				debug_mode.disable
				test_feature (a_context)
				debug_mode.enable
			end
			Result := not precond_violated
		rescue
			debug_mode.enable
			a_file.put_string (associated_class_name)
			a_file.put_string ("%T")
			a_file.put_string (feature_name)
			a_file.put_string ("%T")
			if original_exception = Precondition then
				if (original_recipient_name.is_equal (feature_name)) then
					precond_violated := True
					if tag_name /= Void and tag_name.has_substring ("test_wizard") then
						set_void_target
						a_file.put_string ("void target object")
					else
						set_precondition_violation_exception
						a_file.put_string ("precondition violation")
					end
				else
						-- Assertion violation
					if tag_name /= Void and tag_name.has_substring ("test_wizard") then
						set_void_target
						a_file.put_string ("void target object")
					else
						set_assertion_violation_exception
						a_file.put_string ("assertion violation")
					end
				end
			elseif assertion_violation then
				if tag_name /= Void and tag_name.has_substring ("test_wizard") then
					set_void_target
					a_file.put_string ("void target object")
				else
					set_assertion_violation_exception
					a_file.put_string ("assertion violation")
				end
			else
				set_other_exception
				a_file.put_string ("other exception violation")
			end
			a_file.put_string ("%T")
			if tag_name /= Void then
				a_file.put_string (tag_name)
			end
			a_file.put_string ("%N")
			failure := failure + 1
			retry
		end

feature -- Test feature

	test_feature (a_context: TSGEN_CONTEXT) is
		deferred
		end

feature -- Generate exceptions

	generate_exceptions (a_file: PLAIN_TEXT_FILE) is
			-- Generate results in `a_file'
		require
			file_not_void: a_file /= Void
		do
			a_file.put_string (associated_class_name + "%T")
			a_file.put_string (feature_name + "%T")
			if assertion_violation_exception > 0 or other_exception > 0 or possible_catcall > 0 then
				a_file.put_string ("failed")
			elseif no_exception > 0 then
				a_file.put_string ("passed")
			elseif precondition_violation_exception > 0 then
				a_file.put_string ("no call was valid")
			-- void_target > 0
			else
				a_file.put_string ("could not be tested")
			end
			a_file.put_string ("%T")
			a_file.put_integer (no_exception)
			a_file.put_string ("%T")
			a_file.put_integer (precondition_violation_exception)
			a_file.put_string ("%T")
			a_file.put_integer (assertion_violation_exception)
			a_file.put_string ("%T")
			a_file.put_integer (other_exception)
			a_file.put_string ("%T")
			a_file.put_integer (possible_catcall)
			a_file.put_string ("%T")
			a_file.put_integer (void_target)
			a_file.put_string ("%N")
		end

	generate_results_csv (a_file: PLAIN_TEXT_FILE) is
			-- Generate CSV results in `a_file'
		require
			file_not_void: a_file /= Void
		do
			a_file.put_string (associated_class_name + ", ")
			a_file.put_string (feature_name + ", ")
			if assertion_violation_exception > 0 or other_exception > 0 or possible_catcall > 0 then
				a_file.put_string ("failed")
			elseif no_exception > 0 then
				a_file.put_string ("passed")
			elseif precondition_violation_exception > 0 then
				a_file.put_string ("no call was valid")
			-- void_target > 0
			else
				a_file.put_string ("could not be tested")
			end
			a_file.put_string (", ")
			a_file.put_integer (no_exception)
			a_file.put_string (", ")
			a_file.put_integer (precondition_violation_exception)
			a_file.put_string (", ")
			a_file.put_integer (assertion_violation_exception)
			a_file.put_string (", ")
			a_file.put_integer (other_exception)
			a_file.put_string (", ")
			a_file.put_integer (possible_catcall)
			a_file.put_string (", ")
			a_file.put_integer (void_target)
			a_file.put_string ("%N")
		end

	generate_exceptions_xml (a_file: PLAIN_TEXT_FILE) is
			-- Generate results in xml file `a_file'
		require
			file_not_void: a_file /= Void
		do
			a_file.put_string ("%T<feature>%N")
			if feature_name.has ('<') or feature_name.has ('&') then
				a_file.put_string ("%T%T<name><![CDATA[" + feature_name + "]]></name>%N")
			else
				a_file.put_string ("%T%T<name>" + feature_name + "</name>%N")
			end
			a_file.put_string ("%T%T<class>" + associated_class_name + "</class>%N")
			if assertion_violation_exception > 0 or other_exception > 0 or possible_catcall > 0 then
				a_file.put_string ("%T%T<result>failed</result>%N")
			elseif no_exception > 0 then
				a_file.put_string ("%T%T<result>passed</result>%N")
			elseif precondition_violation_exception > 0 then
				a_file.put_string ("%T%T<result>no call was valid</result>%N")
			-- void_target > 0
			else
				a_file.put_string ("%T%T<result>could not be tested</result>%N")
			end
			a_file.put_string ("%T%T<no_exception>")
			a_file.put_integer (no_exception)
			a_file.put_string ("</no_exception>%N")
			a_file.put_string ("%T%T<precondition_violation>")
			a_file.put_integer (precondition_violation_exception)
			a_file.put_string ("</precondition_violation>%N")
			a_file.put_string ("%T%T<assertion_violation>")
			a_file.put_integer (assertion_violation_exception)
			a_file.put_string ("</assertion_violation>%N")
			a_file.put_string ("%T%T<other_exception>")
			a_file.put_integer (other_exception)
			a_file.put_string ("</other_exception>%N")
			a_file.put_string ("%T%T<possible_catcall>")
			a_file.put_integer (possible_catcall)
			a_file.put_string ("</possible_catcall>%N")
			a_file.put_string ("%T%T<void_target>")
			a_file.put_integer (void_target)
			a_file.put_string ("</void_target>%N")
			a_file.put_string ("%T</feature>%N")
		end

feature -- Name access

	associated_class_name: STRING
			-- Class name

	feature_name: STRING
			-- Feature name

	exhaustiveness: INTEGER
			-- Number of calls

feature -- Name setting

	set_class_name (a_name: STRING) is
			-- Set `associated_class_name' to `a_name'.
		require
			name_not_void: a_name /= Void
			name_not_empty: not a_name.is_empty
		do
			associated_class_name := a_name
		ensure
			name_set: associated_class_name.is_equal (a_name)
		end

	set_feature_name (a_name: STRING) is
			-- Set `feature_name' to `a_name'.
		require
			name_not_void: a_name /= Void
			name_not_empty: not a_name.is_empty
		do
			feature_name := a_name
		ensure
			name_set: feature_name.is_equal (a_name)
		end

	set_exhaustiveness (a_int: INTEGER) is
			-- Set `exhaustiveness' to `a_int'.
		require
			not_negative: a_int >= 0
		do
			exhaustiveness := a_int
		ensure
			exhaustiveness_set: exhaustiveness = a_int
		end

feature -- Exceptions access

	no_exception: INTEGER
			-- Number of calls with no exceptions

	precondition_violation_exception: INTEGER
			-- Number of calls with precondition violations

	assertion_violation_exception: INTEGER
			-- Number of calls with assertion violations

	other_exception: INTEGER
			-- Number of calls with other exceptions

	possible_catcall: INTEGER
			-- Number of calls with possible catcalls

	void_target: INTEGER
			-- Number of calls with void target object

feature -- Exceptions setting

	set_no_exception is
			-- Increment `no_exception'.
		do
			no_exception := no_exception + 1
		end

	set_precondition_violation_exception is
			-- Increment `precondition_violation_exception'.
		do
			precondition_violation_exception := precondition_violation_exception + 1
		end

	set_assertion_violation_exception is
			-- Increment `assertion_violation_exception'.
		do
			assertion_violation_exception := assertion_violation_exception + 1
		end

	set_other_exception is
			-- Increment `other_exception'.
		do
			other_exception := other_exception + 1
		end

	set_possible_catcall is
			-- Increment `possible_catcall'.
		do
			possible_catcall := possible_catcall + 1
		end

	set_void_target is
			-- Increment `void_target'.
		do
			void_target := void_target + 1
		end

end
